/*
 * linux/arch/arm/mach-at91/board-snapper9260-oreo.c
 *
 *  Copyright (C) 2010 Thompson Australia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/init.h>
#include <linux/delay.h>
#include <linux/list.h>

#include <mach/at91sam9_smc.h>
#include <mach/board.h>
#include <mach/gpio.h>

#include <asm/hardware/snapper.h>

#include <video/ssd2119fb.h>

/*
 * SPI devices
 */
#if defined(CONFIG_FB_SSD2119)
#define SSD2119_DC AT91_PIN_PA25
#define SSD2119_RESET AT91_PIN_PC11
#define SSD2119_BUS_ENABLE AT91_PIN_PC6

static void ssd2119_reset(void)
{
	at91_set_gpio_output(SSD2119_RESET, 1);
	at91_set_gpio_value(SSD2119_RESET, 0);
	udelay(10);
	at91_set_gpio_value(SSD2119_RESET, 1);
}

static void ssd2119_set_dc(int dc)
{
        at91_set_gpio_value(SSD2119_DC, dc);
}

static void ssd2119_init(void)
{
        at91_set_gpio_output(SSD2119_DC, 0);
}

static struct ssd2119_platform_data ssd2119_pd = {
        .init = ssd2119_init,
        .reset = ssd2119_reset,
        .set_dc = ssd2119_set_dc,
        .refresh_speed = (HZ / 1),
};
#endif

static struct spi_board_info sn9260_spi_devices[] = {
#if defined(CONFIG_FB_SSD2119)
	{
		.modalias	= "ssd2119fb",
		.chip_select	= 0,
		.bus_num	= 1,
		.max_speed_hz	= 90 * 1000 * 1000,
                .platform_data  = &ssd2119_pd,
	},
#endif
};

static int __init oreo_test_board_init(void)
{
	/* Set the spi bus mux to LCD */
	at91_set_gpio_output(AT91_PIN_PC6, 1);
	at91_add_device_spi(sn9260_spi_devices, ARRAY_SIZE(sn9260_spi_devices));
	return 0;
}

SNAPPER_BASEBOARD(oreo_test_board, oreo_test_board_init);

