/*
 * arch/arm/mach-ep93xx/board-salmon.c
 *
 * Support for Bluewater Systems Salmon baseboard
 * 
 * Author:	Ryan Mallon <ryan@bluewatersys.com>
 * Copyright:	2009 Bluewater Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/irq.h>
#include <linux/i2c.h>
#include <linux/gpio.h>
#include <linux/platform_device.h>

#include <mach/hardware.h>
#include <mach/salmon.h>
#include <asm/hardware/snapper.h>

#include <linux/spi/mmc_spi.h>
#include <linux/mmc/host.h>
#include <mach/spi.h>

#include <linux/i2c/pca953x.h>
#include <linux/spi/ad7927.h>
#include <linux/spi/hi3588.h>

#define SALMON_MODULE_S1	EP93XX_GPIO_LINE_D(0)

int salmon_is_module_s1(void)
{
	return gpio_get_value(SALMON_MODULE_S1);
}

EXPORT_SYMBOL(salmon_is_module_s1);

// TODO sort out a non-sucky way to calculate GPIO base
struct pca953x_platform_data pca9555_0x20 = {
	.gpio_base = 0x40,
};

struct pca953x_platform_data pca9555_0x21 = {
	.gpio_base = 0x50,
};

struct pca953x_platform_data pca9555_0x22 = {
	.gpio_base = 0x60,
};

static struct i2c_board_info __initdata salmon_i2c_devices[] = {
	{
		I2C_BOARD_INFO("pca953x", 0x20),
		.type = "pca9555",
		.platform_data = &pca9555_0x20,
	},
	{
		I2C_BOARD_INFO("pca953x", 0x21),
		.type = "pca9555",
		.platform_data = &pca9555_0x21,
	},
	{
		I2C_BOARD_INFO("pca953x", 0x22),
		.type = "pca9555",
		.platform_data = &pca9555_0x22,
	},
	{
		I2C_BOARD_INFO("ad524x", 0x2f),
		.type = "ad5243",
	},
	{
		I2C_BOARD_INFO("fm31l27x", 0x54),
		.type = "fm31l27x_ram",
	},
	{
		I2C_BOARD_INFO("fm31l27x", 0x6c),
		.type = "fm31l27x_reg",
	},
};

/**
 * Platform configuration for the "ep93xx-spi"
 * EP93XX SPI Driver.
 *
 */
#define SALMON_SPI_NOR_CS	EP93XX_GPIO_LINE_E(5)
#define SALMON_SPI_MMC_CS	EP93XX_GPIO_LINE_F(1)
#define SALMON_SPI_MMC_CD	EP93XX_GPIO_LINE_A(6)
#define SALMON_SPI_MMC_WP	EP93XX_GPIO_LINE_F(3)
#define SALMON_SPI_MMC_EN	EP93XX_GPIO_LINE_F(2)
#define SALMON_SPI_AD7927_CS0   EP93XX_GPIO_LINE_B(5)
#define SALMON_SPI_AD7927_CS1   EP93XX_GPIO_LINE_B(6)	// Note: clashes with Rig200 PCMCIA code
#define SALMON_SPI_AD7927_CS2   EP93XX_GPIO_LINE_B(7)
#define SALMON_SPI_HI3585_CSA   EP93XX_GPIO_LINE_B(2)
#define SALMON_SPI_HI3588_CSB   EP93XX_GPIO_LINE_B(3)
#define SALMON_SPI_HI3588_CSC   EP93XX_GPIO_LINE_B(4)

#define SALMON_SPI_HI3585_RFLAG_A   EP93XX_GPIO_LINE_A(2)
#define SALMON_SPI_HI3588_RFLAG_B   EP93XX_GPIO_LINE_A(3)
#define SALMON_SPI_HI3588_RFLAG_C   EP93XX_GPIO_LINE_A(4)
#define SALMON_SPI_HI3585_TFLAG_A   EP93XX_GPIO_LINE_A(5)

/*
 *	salmon_spi_mmc_init
 *
 *	Card detect irq initialization for the "mmc_spi" driver.
 */
static int
salmon_spi_mmc_init(struct device *pdev,
		    irqreturn_t(*func) (int, void *), void *pdata)
{
	int err;

	err = gpio_request(SALMON_SPI_MMC_CD, "mmc card detect");
	if (err) {
		dev_warn(pdev, "cannot request card detect gpio (%d)\n",
			 err);
		return err;
	}
	err = gpio_request(SALMON_SPI_MMC_WP, "mmc write protect");
	if (err) {
		dev_warn(pdev, "cannot request write protect gpio (%d)\n",
			 err);
		gpio_free(SALMON_SPI_MMC_CD);
		return err;
	}

	gpio_direction_input(SALMON_SPI_MMC_CD);
	gpio_direction_input(SALMON_SPI_MMC_WP);

	// TODO Card Detect is currently unreliable - hardware glitch or software?
	ep93xx_gpio_int_debounce(gpio_to_irq(SALMON_SPI_MMC_CD), 1);

	err = request_irq(gpio_to_irq(SALMON_SPI_MMC_CD), func,
			  IRQ_TYPE_EDGE_BOTH, "mmc card detect", pdata);
	if (err) {
		dev_warn(pdev, "cannot request card detect IRQ (%d)\n",
			 err);
		gpio_free(SALMON_SPI_MMC_WP);
		gpio_free(SALMON_SPI_MMC_CD);
		return err;
	}

	return 0;
}

/*
 *	salmon_spi_mmc_exit
 *
 *	Exit routine for the "mmc_spi" driver.
 */
static void salmon_spi_mmc_exit(struct device *pdev, void *pdata)
{
	free_irq(gpio_to_irq(SALMON_SPI_MMC_CD), pdata);
	gpio_free(SALMON_SPI_MMC_WP);
	gpio_free(SALMON_SPI_MMC_CD);
}

/*
 *	salmon_spi_mmc_get_ro
 *
 *	Read-only switch detect callback for the "mmc_spi" driver.
 *
 *	0 for a read/write card
 *	1 for a read-only card
 *	-ENOSYS when not supported (equal to NULL callback)
 *	or a negative errno value when something bad happened
 */
static int salmon_spi_mmc_get_ro(struct device *pdev)
{
	return !!gpio_get_value(SALMON_SPI_MMC_WP);
}

/*
 *	salmon_spi_mmc_get_cd
 *
 *	Optional card detect callback for the "mmc_spi" driver.
 *
 *	0 for a absent card
 *	1 for a present card
 *	-ENOSYS when not supported (equal to NULL callback)
 *	or a negative errno value when something bad happened
 */
static int salmon_spi_mmc_get_cd(struct device *pdev)
{
	return !gpio_get_value(SALMON_SPI_MMC_CD);
}

/*
 *	salmon_spi_mmc_setpower
 *
 *	Power control callback for the "mmc_spi" driver.
 */
static void
salmon_spi_mmc_setpower(struct device *pdev, unsigned int maskval)
{
	/* FIXME: Add power control to hardware then fix here */
	dev_err(pdev, "%s: %d (stub)\n", __func__, maskval);
}

/* platform_data for the "mmc_spi" driver */
static struct mmc_spi_platform_data salmon_spi_mmc_data = {
	.init = salmon_spi_mmc_init,
	.exit = salmon_spi_mmc_exit,
	.get_ro = salmon_spi_mmc_get_ro,
	.get_cd = salmon_spi_mmc_get_cd,
	.detect_delay = 100,
	.powerup_msecs = 100,
	.ocr_mask = MMC_VDD_32_33 | MMC_VDD_33_34,
	.setpower = salmon_spi_mmc_setpower,
};

/*
 *	Driver setup for the "mmc_spi" driver.
 *
 *	Initialize the chip select when the slave device loads.
 */
static int salmon_spi_mmc_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_MMC_CS, "mmc cs");
	if (err)
		return err;

	err = gpio_request(SALMON_SPI_MMC_EN, "mmc en");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_MMC_CS, 1);
	gpio_direction_output(SALMON_SPI_MMC_EN, 1);

	printk("mmc chip select configured\n");
	return 0;
}

/*
 *	Driver cleanup for the "mmc_spi" driver.
 *
 *	Free the chip select when the slave device unloads.
 */
static void salmon_spi_mmc_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_MMC_CS, 1);
	gpio_direction_input(SALMON_SPI_MMC_CS);
	gpio_free(SALMON_SPI_MMC_CS);

	gpio_set_value(SALMON_SPI_MMC_EN, 1);
	gpio_direction_input(SALMON_SPI_MMC_EN);
	gpio_free(SALMON_SPI_MMC_EN);
}

/*
 *	salmon_spi_mmc_cs_control
 *
 *	Chip-select control for the "mmc_spi" driver.
 */
static void salmon_spi_mmc_hw_cs_control(int active, int activehigh)
{
	// SALMON_SPI_MMC_EN is active low
	if (active)
		gpio_set_value(SALMON_SPI_MMC_EN, 0);
	gpio_set_value(SALMON_SPI_MMC_CS, active ? activehigh : !activehigh);
	if (!active)
		gpio_set_value(SALMON_SPI_MMC_EN, 1);
}

/* controller_data for the "mmc_spi" driver */
static struct ep93xx_spi_chip salmon_spi_mmc_hw = {
	.setup = salmon_spi_mmc_hw_setup,
	.cleanup = salmon_spi_mmc_hw_cleanup,
	.cs_control = salmon_spi_mmc_hw_cs_control,
};

/**
 * Salmon AD7927 devices
 */
static struct ad7927_platform_data salmon_spi_ad7927_pd = {
	.ref_in = 2500,
};

/* AD7927 CS0 */
static int salmon_spi_ad7927_0_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_AD7927_CS0, "ad7927 cs0");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_AD7927_CS0, 1);

	return 0;
}

static void salmon_spi_ad7927_0_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_AD7927_CS0, 1);
	gpio_direction_input(SALMON_SPI_AD7927_CS0);
	gpio_free(SALMON_SPI_AD7927_CS0);
}

static void salmon_spi_ad7927_0_hw_cs_control(int active, int activehigh)
{
	gpio_set_value(SALMON_SPI_AD7927_CS0, active ? activehigh : !activehigh);
}

static struct ep93xx_spi_chip salmon_spi_ad7927_0_hw = {
	.setup = salmon_spi_ad7927_0_hw_setup,
	.cleanup = salmon_spi_ad7927_0_hw_cleanup,
	.cs_control = salmon_spi_ad7927_0_hw_cs_control,
};

/* AD7927 CS1 */
static int salmon_spi_ad7927_1_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_AD7927_CS1, "ad7927 cs1");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_AD7927_CS1, 1);
	return 0;
}

static void salmon_spi_ad7927_1_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_AD7927_CS1, 1);
	gpio_direction_input(SALMON_SPI_AD7927_CS1);
	gpio_free(SALMON_SPI_AD7927_CS1);
}

static void salmon_spi_ad7927_1_hw_cs_control(int active, int activehigh)
{
	gpio_set_value(SALMON_SPI_AD7927_CS1, active ? activehigh : !activehigh);
}

static struct ep93xx_spi_chip salmon_spi_ad7927_1_hw = {
	.setup = salmon_spi_ad7927_1_hw_setup,
	.cleanup = salmon_spi_ad7927_1_hw_cleanup,
	.cs_control = salmon_spi_ad7927_1_hw_cs_control,
};

/* AD7927 CS2 */
static int salmon_spi_ad7927_2_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_AD7927_CS2, "ad7927 cs2");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_AD7927_CS2, 1);

	return 0;
}

static void salmon_spi_ad7927_2_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_AD7927_CS2, 1);
	gpio_direction_input(SALMON_SPI_AD7927_CS2);
	gpio_free(SALMON_SPI_AD7927_CS2);
}

static void salmon_spi_ad7927_2_hw_cs_control(int active, int activehigh)
{
	gpio_set_value(SALMON_SPI_AD7927_CS2, active ? activehigh : !activehigh);
}

static struct ep93xx_spi_chip salmon_spi_ad7927_2_hw = {
	.setup = salmon_spi_ad7927_2_hw_setup,
	.cleanup = salmon_spi_ad7927_2_hw_cleanup,
	.cs_control = salmon_spi_ad7927_2_hw_cs_control,
};

/** HI3585 A **/
static int salmon_spi_hi3585_a_flag_init(struct device *dev,
					 irqreturn_t(*rxfunc) (int,
							       void *),
					 irqreturn_t(*txfunc) (int,
							       void *),
					 void *pdata)
{
	int err;

	err = gpio_request(SALMON_SPI_HI3585_RFLAG_A, "hi3585 rflag a");
	if (err) {
		dev_warn(dev, "cannot request rflag gpio (%d)\n", err);
		return err;
	}
	gpio_direction_input(SALMON_SPI_HI3585_RFLAG_A);

	// Debounce?
	err = request_irq(gpio_to_irq(SALMON_SPI_HI3585_RFLAG_A), rxfunc,
			  IRQ_TYPE_EDGE_BOTH, "hi3585 rflag a", dev);
	if (err) {
		dev_warn(dev, "cannot request rflag IRQ (%d)\n", err);
		gpio_free(SALMON_SPI_HI3585_RFLAG_A);
		return err;
	}

	err = gpio_request(SALMON_SPI_HI3585_TFLAG_A, "hi3585 tflag a");
	if (err) {
		dev_warn(dev, "cannot request tflag gpio (%d)\n", err);
		return err;
	}
	gpio_direction_input(SALMON_SPI_HI3585_TFLAG_A);

	err = request_irq(gpio_to_irq(SALMON_SPI_HI3585_TFLAG_A), txfunc,
			  IRQ_TYPE_EDGE_BOTH, "hi3585 tflag a", dev);
	if (err) {
		dev_warn(dev, "cannot request tflag IRQ (%d)\n", err);
		gpio_free(SALMON_SPI_HI3585_RFLAG_A);
		return err;
	}
	return 0;

}

static int salmon_spi_hi3585_a_flag_exit(struct device *dev, void *pdata)
{
	free_irq(gpio_to_irq(SALMON_SPI_HI3585_RFLAG_A), pdata);
	gpio_free(SALMON_SPI_HI3585_RFLAG_A);
	free_irq(gpio_to_irq(SALMON_SPI_HI3585_TFLAG_A), pdata);
	gpio_free(SALMON_SPI_HI3585_RFLAG_A);
	return 0;
}

static struct hi358x_platform_data salmon_spi_hi3585_a_pd = {
	.model = HI3585,
	.flag_init = salmon_spi_hi3585_a_flag_init,
	.flag_exit = salmon_spi_hi3585_a_flag_exit,
};

static int salmon_spi_hi3585_a_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_HI3585_CSA, "hi3585a cs");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_HI3585_CSA, 1);

	return 0;
}

static void salmon_spi_hi3585_a_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_HI3585_CSA, 1);
	gpio_direction_input(SALMON_SPI_HI3585_CSA);
	gpio_free(SALMON_SPI_HI3585_CSA);
}

static void salmon_spi_hi3585_a_hw_cs_control(int active, int activehigh)
{
	gpio_set_value(SALMON_SPI_HI3585_CSA, active ? activehigh : !activehigh);
}

static struct ep93xx_spi_chip salmon_spi_hi3585_a_hw = {
	.setup = salmon_spi_hi3585_a_hw_setup,
	.cleanup = salmon_spi_hi3585_a_hw_cleanup,
	.cs_control = salmon_spi_hi3585_a_hw_cs_control,
};


/** HI3588 B **/
static int salmon_spi_hi3588_b_flag_init(struct device *dev,
					 irqreturn_t(*rxfunc) (int,
							       void *),
					 irqreturn_t(*txfunc) (int,
							       void *),
					 void *pdata)
{
	int err;

	err = gpio_request(SALMON_SPI_HI3588_RFLAG_B, "hi3588 rflag b");
	if (err) {
		dev_warn(dev, "cannot request rflag gpio (%d)\n", err);
		return err;
	}
	gpio_direction_input(SALMON_SPI_HI3588_RFLAG_B);

	err = request_irq(gpio_to_irq(SALMON_SPI_HI3588_RFLAG_B), rxfunc,
			  IRQ_TYPE_EDGE_BOTH, "hi3588 rflag b", dev);
	if (err) {
		dev_warn(dev, "cannot request rflag IRQ (%d)\n", err);
		gpio_free(SALMON_SPI_HI3588_RFLAG_B);
		return err;
	}
	return 0;

}

static int salmon_spi_hi3588_b_flag_exit(struct device *dev, void *pdata)
{
	free_irq(gpio_to_irq(SALMON_SPI_HI3588_RFLAG_B), pdata);
	gpio_free(SALMON_SPI_HI3588_RFLAG_B);
	return 0;
}

static struct hi358x_platform_data salmon_spi_hi3588_b_pd = {
	.model = HI3588,
	.flag_init = salmon_spi_hi3588_b_flag_init,
	.flag_exit = salmon_spi_hi3588_b_flag_exit,
};

static int salmon_spi_hi3588_b_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_HI3588_CSB, "hi3588b cs");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_HI3588_CSB, 1);

	return 0;
}

static void salmon_spi_hi3588_b_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_HI3588_CSB, 1);
	gpio_direction_input(SALMON_SPI_HI3588_CSB);
	gpio_free(SALMON_SPI_HI3588_CSB);
}

static void salmon_spi_hi3588_b_hw_cs_control(int active, int activehigh)
{
	gpio_set_value(SALMON_SPI_HI3588_CSB, active ? activehigh : !activehigh);
}

static struct ep93xx_spi_chip salmon_spi_hi3588_b_hw = {
	.setup = salmon_spi_hi3588_b_hw_setup,
	.cleanup = salmon_spi_hi3588_b_hw_cleanup,
	.cs_control = salmon_spi_hi3588_b_hw_cs_control,
};

/** HI3588 C **/
static int salmon_spi_hi3588_c_flag_init(struct device *dev,
					 irqreturn_t(*rxfunc) (int,
							       void *),
					 irqreturn_t(*txfunc) (int,
							       void *),
					 void *pdata)
{
	int err;

	err = gpio_request(SALMON_SPI_HI3588_RFLAG_C, "hi3588 rflag c");
	if (err) {
		dev_warn(dev, "cannot request rflag gpio (%d)\n", err);
		return err;
	}
	gpio_direction_input(SALMON_SPI_HI3588_RFLAG_C);

	err = request_irq(gpio_to_irq(SALMON_SPI_HI3588_RFLAG_C), rxfunc,
			  IRQ_TYPE_EDGE_BOTH, "hi3588 rflag c", dev);
	if (err) {
		dev_warn(dev, "cannot request rflag IRQ (%d)\n", err);
		gpio_free(SALMON_SPI_HI3588_RFLAG_C);
		return err;
	}
	return 0;
}

static int salmon_spi_hi3588_c_flag_exit(struct device *dev, void *pdata)
{
	free_irq(gpio_to_irq(SALMON_SPI_HI3588_RFLAG_C), pdata);
	gpio_free(SALMON_SPI_HI3588_RFLAG_C);
	return 0;
}

static struct hi358x_platform_data salmon_spi_hi3588_c_pd = {
	.model = HI3588,
	.flag_init = salmon_spi_hi3588_c_flag_init,
	.flag_exit = salmon_spi_hi3588_c_flag_exit,
};

static int salmon_spi_hi3588_c_hw_setup(void)
{
	int err;
	err = gpio_request(SALMON_SPI_HI3588_CSC, "hi3588c cs");
	if (err)
		return err;

	gpio_direction_output(SALMON_SPI_HI3588_CSC, 1);

	return 0;
}

static void salmon_spi_hi3588_c_hw_cleanup(void)
{
	gpio_set_value(SALMON_SPI_HI3588_CSC, 1);
	gpio_direction_input(SALMON_SPI_HI3588_CSC);
	gpio_free(SALMON_SPI_HI3588_CSC);
}

static void salmon_spi_hi3588_c_hw_cs_control(int active, int activehigh)
{
	gpio_set_value(SALMON_SPI_HI3588_CSC, active ? activehigh : !activehigh);
}

static struct ep93xx_spi_chip salmon_spi_hi3588_c_hw = {
	.setup = salmon_spi_hi3588_c_hw_setup,
	.cleanup = salmon_spi_hi3588_c_hw_cleanup,
	.cs_control = salmon_spi_hi3588_c_hw_cs_control,
};

/* spi bus platform information */
static struct spi_board_info __initdata salmon_spi_board_info[] = {
	/* Chip select 0 is the Snapper CL15 SST25L EEPROM */
	{
	 .modalias		= "ad7927",
	 .platform_data		= &salmon_spi_ad7927_pd,
	 .controller_data 	= &salmon_spi_ad7927_0_hw,
	 .max_speed_hz		= 20000000,
	 .bus_num		= 0,
	 .chip_select		= 1,
	 .mode			= SPI_MODE_1,
	 },
	{
	 .modalias		= "ad7927",
	 .platform_data		= &salmon_spi_ad7927_pd,
	 .controller_data	= &salmon_spi_ad7927_1_hw,
	 .max_speed_hz		= 20000000,
	 .bus_num		= 0,
	 .chip_select		= 2,
	 .mode			= SPI_MODE_1,
	 },
	{
	 .modalias		= "ad7927",
	 .platform_data		= &salmon_spi_ad7927_pd,
	 .controller_data	= &salmon_spi_ad7927_2_hw,
	 .max_speed_hz		= 20000000,
	 .bus_num		= 0,
	 .chip_select		= 3,
	 .mode			= SPI_MODE_1,
	 },
	{
	 .modalias = "hi358x",
	 .platform_data = &salmon_spi_hi3585_a_pd,
	 .controller_data = &salmon_spi_hi3585_a_hw,
	 .max_speed_hz = 2000000,
	 .bus_num = 0,
	 .chip_select = 4,
	 .mode = SPI_MODE_3,	// Clock baseline low, rising edge sampling
	 },
	{
	 .modalias = "hi358x",
	 .platform_data = &salmon_spi_hi3588_b_pd,
	 .controller_data = &salmon_spi_hi3588_b_hw,
	 .max_speed_hz = 2000000,
	 .bus_num = 0,
	 .chip_select = 5,
	 .mode = SPI_MODE_0,
	 },
	{
	 .modalias = "hi358x",
	 .platform_data = &salmon_spi_hi3588_c_pd,
	 .controller_data = &salmon_spi_hi3588_c_hw,
	 .max_speed_hz = 2000000,
	 .bus_num = 0,
	 .chip_select = 6,
	 .mode = SPI_MODE_0,
	 },
	{
	 .modalias = "mmc_spi",
	 .platform_data = &salmon_spi_mmc_data,
	 .controller_data = &salmon_spi_mmc_hw,
	 .max_speed_hz           = 20000000,
	 .bus_num = 0,
	 .chip_select = 7,
	 .mode = SPI_MODE_3,
	 },
};

/* The switch MDIO interface is the same one used by the ethernet MAC */
static struct resource salmon_mv88e6061_resource[] = {
    {
	.start = EP93XX_ETHERNET_PHYS_BASE,
	.end = EP93XX_ETHERNET_PHYS_BASE + 0xffff,
	.flags = IORESOURCE_MEM,
    }
};

static struct platform_device salmon_mv88e6061_device= {
    .name = "mv88e6061-switch",
    .id = -1,
    .num_resources = 1,
    .resource = salmon_mv88e6061_resource,
};

static int __init salmon_init(void)
{
	int err;

	err = gpio_request(SALMON_MODULE_S1, "Module S1");
	if (err)
		return err;
	gpio_direction_input(SALMON_MODULE_S1);

        /* Turn on the UARTS ports */
	ep93xx_devcfg_set_bits(EP93XX_SYSCON_DEVICE_CONFIG_UART3_ENABLE |
			       EP93XX_SYSCON_DEVICE_CONFIG_UART2_ENABLE |
			       EP93XX_SYSCON_DEVICE_CONFIG_UART1_ENABLE);

	// Quash CS for all the drivers we're not using right now      
	gpio_request(SALMON_SPI_NOR_CS, "nor en");
	gpio_direction_output(SALMON_SPI_NOR_CS, 1);
	gpio_set_value(SALMON_SPI_NOR_CS, 1);

#if 0
	// TODO need some kind of framework to make SPI safe from devices not being 
	// configured on the bus
	gpio_request(SALMON_SPI_MMC_CS, "mmc cs");
	gpio_direction_output(SALMON_SPI_MMC_CS, 1);
	gpio_set_value(SALMON_SPI_MMC_CS, 1);

	gpio_request(SALMON_SPI_AD7927_CS0, "adc0 cs");
	gpio_direction_output(SALMON_SPI_AD7927_CS0, 1);
	gpio_set_value(SALMON_SPI_AD7927_CS0, 1);

	gpio_request(SALMON_SPI_AD7927_CS1, "adc1 cs");
	gpio_direction_output(SALMON_SPI_AD7927_CS1, 1);
	gpio_set_value(SALMON_SPI_AD7927_CS1, 1);

	gpio_request(SALMON_SPI_AD7927_CS2, "adc2 cs");
	gpio_direction_output(SALMON_SPI_AD7927_CS2, 1);
	gpio_set_value(SALMON_SPI_AD7927_CS2, 1);

	gpio_request(SALMON_SPI_HI3585_CSA, "hi3585a cs");
	gpio_direction_output(SALMON_SPI_HI3585_CSA, 1);
	gpio_set_value(SALMON_SPI_HI3585_CSA, 1);

	gpio_request(SALMON_SPI_HI3588_CSB, "hi3588b cs");
	gpio_direction_output(SALMON_SPI_HI3588_CSB, 1);
	gpio_set_value(SALMON_SPI_HI3588_CSB, 1);

	gpio_request(SALMON_SPI_HI3588_CSC, "hi3588c cs");
	gpio_direction_output(SALMON_SPI_HI3588_CSC, 1);
	gpio_set_value(SALMON_SPI_HI3588_CSC, 1);
#endif
/****************************************************************************************/

	ep93xx_register_spi(salmon_spi_board_info,
			    ARRAY_SIZE(salmon_spi_board_info));
	ep93xx_register_i2c(salmon_i2c_devices,
			    ARRAY_SIZE(salmon_i2c_devices));
	
	platform_device_register (&salmon_mv88e6061_device);
	
	return 0;
}

SNAPPER_BASEBOARD(salmon, salmon_init);
