/*
 * Simple SPI SSD2119 display driver
 *
 * Author: Carl Smith <carl@bluewatersys.com>
 *
 * Copyright (c) 2009, Bluewater Systems Ltd
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#ifndef __SSD2119FB_H__
#define __SSD2119FB_H__

struct ssd2119_platform_data {
        void (*init)(void);
        void (*reset)(void);
        void (*set_dc)(int level);
        int refresh_speed;
};

/* Frame buffer info */
struct ssd2119_fb_info {
	struct fb_info *	info;
	struct spi_device	*spi;
	struct timer_list	auto_update_timer;
        dma_addr_t              map_dma;
        u_char *                map_cpu;
        uint32_t                map_size;
        struct ssd2119_platform_data *pd;
        struct work_struct      task;
};

#define SSD2119_OSC_START_REG         0x00
#define SSD2119_OUTPUT_CTRL_REG       0x01
#define SSD2119_LCD_DRIVE_AC_CTRL_REG 0x02
#define SSD2119_DISPLAY_CTRL_REG      0x07
#define SSD2119_PWR_CTRL_2_REG        0x0C
#define SSD2119_PWR_CTRL_3_REG        0x0D
#define SSD2119_PWR_CTRL_4_REG        0x0E
#define SSD2119_SLEEP_MODE_REG        0x10
#define SSD2119_ENTRY_MODE_REG        0x11
#define SSD2119_RAM_DATA_REG          0x22
#define SSD2119_V_RAM_POS_REG         0x44
#define SSD2119_H_RAM_START_REG       0x45
#define SSD2119_H_RAM_END_REG         0x46
#define SSD2119_X_RAM_ADDR_REG        0x4E
#define SSD2119_Y_RAM_ADDR_REG        0x4F

#define LCD_HEIGHT 240
#define LCD_WIDTH 320

#endif /* __SSD2119FB_H__ */
